from odoo import api, models, fields, _
from odoo.exceptions import UserError

ODOO_TEMPLATES = {
    'payment_receipt': ['en', ],
    'invoice': ['en', ],
    'sale_order_ship': ['en', ],
    'event_ticket': ['en', ],
    'event_reminder': ['en', ],
    'payment_link': ['en', ],
    'pos_receipt': ['en', ],
    'sale': ['en', ],
}


class WhatsAppTemplate(models.Model):
    _inherit = 'whatsapp.template'

    is_demo_account = fields.Boolean(compute='_compute_is_demo_account')

    @api.model_create_multi
    def create(self, vals):
        odoo_account = self.env.ref('saas_whatsapp.whatsapp_meta_account_odoo_demo_account', False)
        if odoo_account and odoo_account.phone_uid != 'odoo_demo_account':
            odoo_account = False  # Record has been replaced with real account
        for vals_dict in vals:
            if (odoo_account and vals_dict.get('template_name')
                    and vals_dict.get('lang_code') and not vals_dict.get('wa_account_id')):
                if vals_dict['lang_code'] in ODOO_TEMPLATES.get(vals_dict['template_name'], []):
                    vals_dict['wa_account_id'] = odoo_account.id
                    vals_dict['status'] = 'approved'
        return super().create(vals)

    @api.depends('wa_account_id')
    def _compute_is_demo_account(self):
        for t in self:
            t.is_demo_account = t.wa_account_id and t.wa_account_id.phone_uid == 'odoo_demo_account'

    def open_whatsapp_config(self):
        return {
            'type': 'ir.actions.act_window',
            'name': 'WhatsApp Business Account',
            'view_mode': 'form',
            'res_model': 'whatsapp.account',
            'res_id': self.env.ref('saas_whatsapp.whatsapp_meta_account_odoo_demo_account').id,
            'target': 'current',
        }

    def button_reset_to_draft(self):
        for tmpl in self:
            if tmpl.is_demo_account:
                self.raise_demo_warning()
        return super().button_reset_to_draft()

    def _get_send_template_vals(self, record, free_text_json, attachment=False):
        if free_text_json and self.wa_account_id and self.wa_account_id.phone_uid == 'odoo_demo_account':
            self.raise_demo_warning('[TEXT_FIELD]')
        return super()._get_send_template_vals(record, free_text_json, attachment=attachment)

    @staticmethod
    def raise_demo_warning(error=''):
        raise UserError(_("""
            You are currently using a limited demo WhatsApp Business Account.
            With this demo, you can only utilize predefined templates.
            To unlock the full range of features, create your own account.
        """) + "[%s]" % error)
